@@ -90,8 +90,13 @@ int mapMods_Keys(int modFlags) {
         KMOD_CAPS,
     };
     int mapped = 0;
-    /* Treat capslock as a modifier key. */
-    modFlags |= (capsLockDown_ ? KMOD_CAPS : 0);
+    if (prefs_App()->capsLockKeyModifier) {
+        /* Treat capslock as a modifier key. */
+        modFlags |= (capsLockDown_ ? KMOD_CAPS : 0);
+    }
+    else {
+        modFlags &= ~KMOD_CAPS;
+    }
     for (int i = 0; i < max_ModMap; ++i) {
         if (modFlags & bits[i]) {
             mapped |= bits[modMap_[i]];
@@ -194,8 +199,8 @@ enum iBindFlag {
 /* TODO: This indirection could be used for localization, although all UI strings
    would need to be similarly handled. */
 static const struct { int id; iMenuItem bind; int flags; } defaultBindings_[] = {
-    { 1,  { "${keys.top}",                  SDLK_HOME, 0,                   "scroll.top"                        }, 0 },
-    { 2,  { "${keys.bottom}",               SDLK_END, 0,                    "scroll.bottom"                     }, 0 },
+    { 1,  { "${LC:keys.top}",               SDLK_HOME, 0,                   "scroll.top"                        }, 0 },
+    { 2,  { "${LC:keys.bottom}",            SDLK_END, 0,                    "scroll.bottom"                     }, 0 },
     { 10, { "${keys.scroll.up}",            SDLK_UP, 0,                     "scroll.step arg:-1"                }, argRepeat_BindFlag },
     { 11, { "${keys.scroll.down}",          SDLK_DOWN, 0,                   "scroll.step arg:1"                 }, argRepeat_BindFlag },
     { 22, { "${keys.scroll.halfpage.up}",   SDLK_SPACE, KMOD_SHIFT,         "scroll.page arg:-1"                }, argRepeat_BindFlag },
@@ -208,6 +213,7 @@ static const struct { int id; iMenuItem bind; int flags; } defaultBindings_[] =
     { 33, { "${keys.root}",                 navigateRoot_KeyShortcut,       "navigate.root"                     }, 0 },
     { 35, { "${keys.reload}",               reload_KeyShortcut,             "document.reload"                   }, 0 },
     { 36, { "${LC:menu.openlocation}",      SDLK_l, KMOD_PRIMARY,           "navigate.focus"                    }, 0 },
+    { 37, { "${keys.bang}",                 SDLK_1, KMOD_SHIFT,             "navigate.focus text:!"             }, 0 },
     { 41, { "${keys.link.modkey}",          SDLK_LALT, 0,                   "document.linkkeys arg:0"           }, argRelease_BindFlag },
     { 42, { "${keys.link.homerow}",         'f', 0,                         "document.linkkeys arg:1"           }, 0 },
     { 45, { "${keys.link.homerow.newtab}",  'f', KMOD_SHIFT,                "document.linkkeys arg:1 newtab:1"  }, 0 },
@@ -226,10 +232,10 @@ static const struct { int id; iMenuItem bind; int flags; } defaultBindings_[] =
 #if !defined (iPlatformApple) /* Ctrl-Cmd-F on macOS */
     { 73, { "${keys.fullscreen}",           SDLK_F11, 0,                    "window.fullscreen"                 }, 0 },
 #endif
-    { 76, { "${keys.tab.new}",              newTab_KeyShortcut,             "tabs.new"                          }, 0 },
+    { 76, { "${keys.tab.new}",              newTab_KeyShortcut,             "tabs.new append:1"                 }, 0 },
     { 77, { "${keys.tab.close}",            closeTab_KeyShortcut,           "tabs.close"                        }, 0 },
     { 78, { "${keys.tab.close.other}",      SDLK_w, KMOD_SECONDARY,         "tabs.close toleft:1 toright:1"     }, 0 },
-    { 79, { "${LC:menu.reopentab}",         SDLK_t, KMOD_SECONDARY,         "tabs.new reopen:1"                 }, 0 },        
+    { 79, { "${LC:menu.reopentab}",         SDLK_t, KMOD_SECONDARY,         "tabs.new reopen:1"                 }, 0 },
     { 80, { "${keys.tab.prev}",             prevTab_KeyShortcut,            "tabs.prev"                         }, 0 },
     { 81, { "${keys.tab.next}",             nextTab_KeyShortcut,            "tabs.next"                         }, 0 },
     { 84, { "${LC:menu.movetab.left}",      moveTabLeft_KeyShortcut,        "tabs.move arg:-1"                  }, 0 },
@@ -252,7 +258,7 @@ static const struct { int id; iMenuItem bind; int flags; } defaultBindings_[] =
     { 125,{ "${keys.pageinfo}",             pageInfo_KeyShortcut,           "document.info"                     }, 0 },
     { 126,{ "${keys.sitespec}",             ',', KMOD_SECONDARY,            "document.sitespec"                 }, 0 },
     { 130,{ "${keys.input.precedingline}",  SDLK_v, KMOD_SECONDARY,         "input.precedingline"               }, 0 },
-    { 140,{ "${keys.identmenu}",            identityMenu_KeyShortcut,       "identmenu.open focus:1"            }, 0 },          
+    { 140,{ "${keys.identmenu}",            identityMenu_KeyShortcut,       "identmenu.open focus:1"            }, 0 },
     { 200,{ "${keys.menubar.focus}",        menuBar_KeyShortcut,            "menubar.focus"                     }, 0 },
     { 205,{ "${keys.contextmenu}",          '/', 0,                         "contextkey"                        }, 0 },
     /* The following cannot currently be changed (built-in duplicates). */
@@ -498,3 +504,12 @@ const iPtrArray *list_Keys(void) {
     }
     return list;
 }
+
+iBool isDown_Keys(const iBinding *binding) {
+    int numKeys = 0;
+    const uint8_t *keys = SDL_GetKeyboardState(&numKeys);
+    const int scancode = SDL_GetScancodeFromKey(binding->key);
+    const int scancode2 = (scancode == SDL_SCANCODE_RALT ? SDL_SCANCODE_LALT :
+                           scancode == SDL_SCANCODE_LALT ? SDL_SCANCODE_RALT : scancode);
+    return (scancode < numKeys && keys[scancode]) || (scancode2 < numKeys && keys[scancode2]);
+}
